from typing import List, Dict, Any
import supervisely as sly
from supervisely import Annotation, VideoAnnotation
import numpy as np

class BaseTracker:

    def __init__(self, settings: dict = None, device: str = None):
        import torch  # pylint: disable=import-error
        self.settings = settings or {}
        auto_device = "cuda" if torch.cuda.is_available() else "cpu"
        settings_device = self.settings.get("device")
        
        if settings_device is not None:
            if settings_device == "auto":
                self.device = auto_device
            else:
                self.device = settings_device
        else:
            self.device = device if device is not None else auto_device
                
        self._validate_device()


    def update(self, frame: np.ndarray, annotation: Annotation) -> List[Dict[str, Any]]:
        raise NotImplementedError("This method should be overridden by subclasses.")

    def reset(self) -> None:
        """Reset tracker state."""
        pass    
    
    def track(self, frames: List[np.ndarray], annotations: List[Annotation]) -> VideoAnnotation:
        raise NotImplementedError("This method should be overridden by subclasses.")
    
    @property
    def video_annotation(self) -> VideoAnnotation:
        """Return the accumulated VideoAnnotation."""
        raise NotImplementedError("This method should be overridden by subclasses.")

    def _validate_device(self) -> None:
        if self.device != 'cpu' and not self.device.startswith('cuda'):
            raise ValueError(
                f"Invalid device '{self.device}'. Supported devices are 'cpu' or 'cuda'."
            )



import os
from typing import Any, Dict, List

from fastapi import Request, Response, status

from supervisely import Api, PointcloudAnnotation, PointcloudFigure, PointcloudObject
from supervisely._utils import rand_str
from supervisely.annotation.label import Label
from supervisely.annotation.tag import Tag
from supervisely.app.content import get_data_dir
from supervisely.geometry.cuboid_3d import Cuboid3d
from supervisely.io.fs import silent_remove
from supervisely.nn.inference.inference import Inference
from supervisely.nn.prediction_dto import PredictionCuboid3d
from supervisely.pointcloud_annotation.pointcloud_object_collection import (
    PointcloudObjectCollection,
)
from supervisely.sly_logger import logger


class ObjectDetection3D(Inference):
    def get_info(self) -> dict:
        info = super().get_info()
        info["task type"] = "object detection 3d"
        info["videos_support"] = False
        info["async_video_inference_support"] = False
        info["tracking_on_videos_support"] = False
        info["async_image_inference_support"] = False

        # recommended parameters:
        # info["model_name"] = ""
        # info["checkpoint_name"] = ""
        # info["pretrained_on_dataset"] = ""
        # info["device"] = ""
        return info

    def _get_obj_class_shape(self):
        return Cuboid3d

    def _create_label(self, dto: PredictionCuboid3d):
        raise NotImplementedError()

    def predict(self, pcd_path: str, settings: Dict[str, Any]) -> List[PredictionCuboid3d]:
        raise NotImplementedError("Have to be implemented in child class")

    def predict_raw(self, pcd_path: str, settings: Dict[str, Any]) -> List[PredictionCuboid3d]:
        raise NotImplementedError(
            "Have to be implemented in child class If sliding_window_mode is 'advanced'."
        )

    def _inference_pointcloud_id(self, api: Api, pointcloud_id: int, settings: Dict[str, Any]):
        # 1. download pointcloud
        pcd_path = os.path.join(get_data_dir(), rand_str(10) + ".pcd")
        api.pointcloud.download_path(pointcloud_id, pcd_path)
        # 2. predict
        prediction = self.predict(pcd_path, settings)
        # 3. clean up
        silent_remove(pcd_path)
        return prediction

    def annotation_from_prediction(
        self,
        predictions: List[PredictionCuboid3d],
    ) -> PointcloudAnnotation:
        model_meta = self._model_meta
        objects = []
        figures = []
        for prediction in predictions:
            class_name = prediction.class_name
            geometry = prediction.cuboid_3d
            object = PointcloudObject(model_meta.get_obj_class(class_name))
            figure = PointcloudFigure(object, geometry)
            objects.append(object)
            figures.append(figure)
        objects = PointcloudObjectCollection(objects)
        annotation = PointcloudAnnotation(objects, figures)
        return annotation

    def raw_results_from_prediction(
        self, prediction: List[PredictionCuboid3d]
    ) -> List[Dict[str, Any]]:
        results = []
        for pred in prediction:
            detection_name = pred.class_name
            translation = list(pred.cuboid_3d.position.to_json().values())
            size = list(pred.cuboid_3d.dimensions.to_json().values())
            rotation_z = pred.cuboid_3d.rotation.z
            velocity = [0, 0]  # Is not supported now
            detection_score = pred.score
            results.append(
                {
                    "detection_name": detection_name,
                    "translation": translation,
                    "size": size,
                    "rotation": rotation_z,
                    "velocity": velocity,
                    "detection_score": detection_score,
                }
            )
        return results

    def serve(self):
        super().serve()
        server = self._app.get_server()

        @server.post("/inference_pointcloud_id")
        def inference_pointcloud_id(response: Response, request: Request):
            logger.debug(
                f"inference_pointcloud_id: state=",
                extra={**request.state.state, "api_token": "***"},
            )
            state = request.state.state
            api: Api = request.state.api
            settings = self._get_inference_settings(state)
            prediction = self._inference_pointcloud_id(api, state["pointcloud_id"], settings)
            annotation = self.annotation_from_prediction(prediction)
            raw_results = self.raw_results_from_prediction(prediction)
            result = {
                "results": {
                    "annotation": annotation.to_json(),
                    "raw_results": raw_results,
                }
            }  # This format is used in "Apply 3D Detection to Pointcloud Project" app.
            return result

        @server.post("/inference_pointcloud_ids")
        def inference_pointcloud_ids(response: Response, request: Request):
            logger.debug(
                f"inference_pointcloud_ids: state=",
                extra={**request.state.state, "api_token": "***"},
            )
            state = request.state.state
            api: Api = request.state.api
            settings = self._get_inference_settings(state)
            annotations = []
            for pcd_id in state["pointcloud_ids"]:
                prediction = self._inference_pointcloud_id(api, pcd_id, settings)
                annotation = self.annotation_from_prediction(prediction)
                annotations.append(annotation.to_json())
            return annotations



import os
from typing import List, Any, Dict

from supervisely.api.api import Api
from supervisely.app.widgets import Button, Container, Card, Text, GridGallery


class Preview:
    title = "Preview"
    description = "Preview the model output"
    lock_message = None

    def __init__(self, api: Api, static_dir: str):
        # Init Step
        self.api = api
        self.display_widgets: List[Any] = []
        self.static_dir = static_dir
        self.inference_settings = None
        # -------------------------------- #

        # Init Base Widgets
        self.validator_text = None
        self.button = None
        self.container = None
        self.card = None
        # -------------------------------- #

        # Init Step Widgets
        self.gallery = None
        # -------------------------------- #

        # Preview Directory
        self.preview_dir = os.path.join(self.static_dir, "preview")
        os.makedirs(self.preview_dir, exist_ok=True)
        self.preview_path = os.path.join(self.preview_dir, "preview.jpg")
        self.peview_url = f"/static/preview/preview.jpg"
        # ----------------------------------- #

        # Preview Widget
        self.gallery = GridGallery(
            2,
            sync_views=True,
            enable_zoom=True,
            resize_on_zoom=True,
            empty_message="Click 'Preview' to see the model output.",
        )
        # Add widgets to display ------------ #
        self.display_widgets.extend([self.gallery])
        # ----------------------------------- #

        # Base Widgets
        self.validator_text = Text("")
        self.validator_text.hide()
        self.button = Button("Preview", icon="zmdi zmdi-eye")
        # Add widgets to display ------------ #
        self.display_widgets.extend([self.validator_text, self.button])
        # ----------------------------------- #

        # Card Layout
        self.container = Container(self.display_widgets)
        self.card = Card(
            title="Preview",
            content=self.container,
            lock_message=self.lock_message,
        )
        self.card.lock()
        # ----------------------------------- #

        @self.button.click
        def button_click():
            self.run_preview()

    @property
    def widgets_to_disable(self) -> list:
        return [self.gallery]

    def load_from_json(self, data: Dict[str, Any]) -> None:
        return

    def get_settings(self) -> Dict[str, Any]:
        return {
            "preview_path": self.preview_path,
            "preview_url": self.peview_url,
            "inference_settings": self.inference_settings,
        }

    def validate_step(self) -> bool:
        return True

    def run_preview(self) -> None:
        raise NotImplementedError(
            "run_preview must be implemented by subclasses or injected at runtime"
        )



from typing import Literal
from supervisely.app.widgets import Widget, Progress
from supervisely.app.content import DataJson


class CircleProgress(Widget):
    def __init__(self, progress: Progress, widget_id=None):
        self.progress = progress
        super().__init__(widget_id=widget_id, file_path=__file__)

    def get_json_data(self):
        return {
            "status": None,
        }

    def get_json_state(self):
        return {}

    def set_status(self, status: Literal["success", "exception", "none"]):
        if status == "none":
            status = None
        DataJson()[self.widget_id]["status"] = status
        DataJson().send_changes()



import time
import uuid
from pathlib import Path
from typing import Any, BinaryIO, Dict, List, Optional

import numpy as np
from pydantic import ValidationError

from supervisely.annotation.annotation import Annotation
from supervisely.annotation.label import Geometry, Label, LabelingStatus
from supervisely.annotation.obj_class import ObjClass
from supervisely.api.api import Api
from supervisely.api.module_api import ApiField
from supervisely.api.video.video_figure_api import FigureInfo
from supervisely.geometry.helpers import deserialize_geometry
from supervisely.geometry.rectangle import Rectangle
from supervisely.imaging import image as sly_image
from supervisely.nn.inference.inference import Uploader
from supervisely.nn.inference.inference_request import InferenceRequest
from supervisely.nn.inference.tracking.base_tracking import BaseTracking
from supervisely.nn.inference.tracking.tracker_interface import (
    TrackerInterface,
    TrackerInterfaceV2,
)
from supervisely.nn.inference.uploader import Uploader
from supervisely.nn.prediction_dto import Prediction, PredictionBBox
from supervisely.sly_logger import logger


class BBoxTracking(BaseTracking):
    def _deserialize_geometry(self, data: dict):
        geometry_type_str = data["type"]
        geometry_json = data["data"]
        return deserialize_geometry(geometry_type_str, geometry_json)

    def _track(
        self,
        api: Api,
        context: dict,
        inference_request: InferenceRequest,
    ):
        video_interface = TrackerInterface(
            context=context,
            api=api,
            load_all_frames=False,
            frame_loader=self.cache.download_frame,
            frames_loader=self.cache.download_frames,
            should_notify=True,
        )

        range_of_frames = [
            video_interface.frames_indexes[0],
            video_interface.frames_indexes[-1],
        ]

        if self.cache.is_persistent:
            self.cache.run_cache_task_manually(
                api,
                None,
                video_id=video_interface.video_id,
            )
        else:
            # if cache is not persistent, run cache task for range of frames
            self.cache.run_cache_task_manually(
                api,
                [range_of_frames],
                video_id=video_interface.video_id,
            )

        def _upload_f(items: List):
            video_interface.add_object_geometries_on_frames(*list(zip(*items)), notify=False)
            inference_request.done(len(items))

        def _notify_f(items: List):
            frame_range = [
                min(frame_index for (_, _, frame_index) in items),
                max(frame_index for (_, _, frame_index) in items),
            ]
            pos_inc = inference_request.progress.current - video_interface.global_pos

            video_interface._notify(
                pos_increment=pos_inc,
                fstart=frame_range[0],
                fend=frame_range[1],
                task=inference_request.stage,
            )

        def _exception_handler(exception: Exception):
            try:
                raise exception
            except Exception:
                api.logger.error(f"Error: {str(exception)}", exc_info=True)
            video_interface._notify(True, task="Stop tracking due to an error")
            raise exception

        api.logger.info("Start tracking.")
        total_progress = video_interface.frames_count * len(video_interface.figure_ids)
        inference_request.set_stage(InferenceRequest.Stage.INFERENCE, 0, total_progress)
        with Uploader(
            upload_f=_upload_f,
            notify_f=_notify_f,
            exception_handler=_exception_handler,
            logger=api.logger,
        ) as uploader:
            for fig_id, obj_id in zip(
                video_interface.geometries.keys(),
                video_interface.object_ids,
            ):
                init = False
                for _ in video_interface.frames_loader_generator():
                    geom = video_interface.geometries[fig_id]
                    if not isinstance(geom, Rectangle):
                        raise TypeError(f"Tracking does not work with {geom.geometry_name()}.")

                    imgs = video_interface.frames
                    target = PredictionBBox(
                        "",  # TODO: can this be useful?
                        [geom.top, geom.left, geom.bottom, geom.right],
                        None,
                    )

                    if not init:
                        self.initialize(imgs[0], target)
                        init = True

                    geometry = self.predict(
                        rgb_image=imgs[-1],
                        prev_rgb_image=imgs[0],
                        target_bbox=target,
                        settings=self.custom_inference_settings_dict,
                    )
                    sly_geometry = self._to_sly_geometry(geometry)

                    uploader.put([(sly_geometry, obj_id, video_interface._cur_frames_indexes[-1])])

                    if inference_request.is_stopped() or video_interface.global_stop_indicatior:
                        api.logger.info(
                            "Inference request stopped.",
                            extra={"inference_request_uuid": inference_request.uuid},
                        )
                        video_interface._notify(True, task="Stop tracking")
                        return
                    if uploader.has_exception():
                        exception = uploader.exception
                        if not isinstance(exception, Exception):
                            raise RuntimeError(
                                f"Uploader exception is not an instance of Exception: {str(exception)}"
                            )
                        raise uploader.exception

                api.logger.info(
                    f"Figure #{fig_id} tracked.",
                    extra={
                        "figure_id": fig_id,
                        "object_id": obj_id,
                        "inference_request_uuid": inference_request.uuid,
                    },
                )
            api.logger.info(
                "Finished tracking.", extra={"inference_request_uuid": inference_request.uuid}
            )
            video_interface._notify(True, task="Finished tracking")

    def _track_api(self, api: Api, context: dict, inference_request: InferenceRequest):
        track_t = time.monotonic()
        # unused fields:
        context["trackId"] = "auto"
        context["objectIds"] = []
        context["figureIds"] = []
        if "direction" not in context:
            context["direction"] = "forward"

        input_bboxes: list = context["input_geometries"]

        video_interface = TrackerInterface(
            context=context,
            api=api,
            load_all_frames=False,
            frame_loader=self.cache.download_frame,
            frames_loader=self.cache.download_frames,
            should_notify=False,
        )

        range_of_frames = [
            video_interface.frames_indexes[0],
            video_interface.frames_indexes[-1],
        ]

        if self.cache.is_persistent:
            # if cache is persistent, run cache task for whole video
            self.cache.run_cache_task_manually(
                api,
                None,
                video_id=video_interface.video_id,
            )
        else:
            # if cache is not persistent, run cache task for range of frames
            self.cache.run_cache_task_manually(
                api,
                [range_of_frames],
                video_id=video_interface.video_id,
            )

        predictions = []
        frames_n = video_interface.frames_count
        box_n = len(input_bboxes)
        geom_t = time.monotonic()

        inference_request.set_stage(InferenceRequest.Stage.INFERENCE, 0, frames_n * box_n)
        api.logger.info(
            "Start tracking.",
            extra={
                "video_id": video_interface.video_id,
                "frame_range": range_of_frames,
                "geometries_count": box_n,
                "frames_count": frames_n,
                "request_uuid": inference_request.uuid,
            },
        )
        for box_i, input_geom in enumerate(input_bboxes, 1):
            input_bbox = input_geom["data"]
            bbox = Rectangle.from_json(input_bbox)
            predictions_for_object = []
            init = False
            frame_t = time.monotonic()
            for frame_i, _ in enumerate(video_interface.frames_loader_generator(), 1):
                imgs = video_interface.frames
                target = PredictionBBox(
                    "",  # TODO: can this be useful?
                    [bbox.top, bbox.left, bbox.bottom, bbox.right],
                    None,
                )

                if not init:
                    self.initialize(imgs[0], target)
                    init = True

                geometry = self.predict(
                    rgb_image=imgs[-1],
                    prev_rgb_image=imgs[0],
                    target_bbox=target,
                    settings=self.custom_inference_settings_dict,
                )
                sly_geometry = self._to_sly_geometry(geometry)

                predictions_for_object.append(
                    {"type": sly_geometry.geometry_name(), "data": sly_geometry.to_json()}
                )
                inference_request.done()
                api.logger.debug(
                    "Frame processed. Geometry: [%d / %d]. Frame: [%d / %d]",
                    box_i,
                    box_n,
                    frame_i,
                    frames_n,
                    extra={
                        "geometry_index": box_i,
                        "frame_index": frame_i,
                        "processing_time": time.monotonic() - frame_t,
                        "inference_request_uuid": inference_request.uuid,
                    },
                )
                frame_t = time.monotonic()

            predictions.append(predictions_for_object)
            api.logger.info(
                "Geometry processed. Progress: [%d / %d]",
                box_i,
                box_n,
                extra={
                    "geometry_index": box_i,
                    "processing_time": time.monotonic() - geom_t,
                    "inference_request_uuid": inference_request.uuid,
                },
            )
            geom_t = time.monotonic()

        # predictions must be NxK bboxes: N=number of frames, K=number of objects
        predictions = list(map(list, zip(*predictions)))
        api.logger.info(
            "Tracking finished.",
            extra={
                "tracking_time": time.monotonic() - track_t,
                "inference_request_uuid": inference_request.uuid,
            },
        )
        inference_request.final_result = predictions

    def _inference(self, frames: List[np.ndarray], geometries: List[Geometry], settings: dict):
        updated_settings = {
            **self.custom_inference_settings_dict,
            **settings,
        }
        results = [[] for _ in range(len(frames) - 1)]
        for geometry in geometries:
            if not isinstance(geometry, Rectangle):
                raise TypeError(f"Tracking does not work with {geometry.geometry_name()}.")
            target = PredictionBBox(
                "",
                [geometry.top, geometry.left, geometry.bottom, geometry.right],
                None,
            )
            self.initialize(frames[0], target)
            for i in range(len(frames) - 1):
                pred_geometry = self.predict(
                    rgb_image=frames[i + 1],
                    prev_rgb_image=frames[i],
                    target_bbox=target,
                    settings=updated_settings,
                )
                sly_pred_geometry = self._to_sly_geometry(pred_geometry)
                results[i].append(
                    {"type": Rectangle.geometry_name(), "data": sly_pred_geometry.to_json()}
                )
        return results

    def _track_async(self, api: Api, context: dict, inference_request: InferenceRequest):
        tracker_interface = TrackerInterfaceV2(api, context, self.cache)
        frames_count = tracker_interface.frames_count
        figures = tracker_interface.figures
        frame_range = [
            tracker_interface.frame_indexes[0],
            tracker_interface.frame_indexes[-1],
        ]
        frame_range_asc = [min(frame_range), max(frame_range)]
        progress_total = frames_count * len(figures)

        def _upload_f(items: List[FigureInfo]):
            inference_request.add_results(items)
            inference_request.done(len(items))

        def _notify_f(items: List[FigureInfo]):
            frame_range = [
                min(item.frame_index for item in items),
                max(item.frame_index for item in items),
            ]
            tracker_interface.notify_progress(
                inference_request.progress.current, inference_request.progress.total, frame_range
            )

        def _exception_handler(exception: Exception):
            api.logger.error(f"Error saving predictions: {str(exception)}", exc_info=True)
            tracker_interface.notify_progress(
                inference_request.progress.current,
                inference_request.progress.current,
                frame_range_asc,
            )
            tracker_interface.notify_error(exception)
            raise Exception

        api.logger.info("Start tracking.", extra={"inference_request_uuid": inference_request.uuid})
        inference_request.set_stage(InferenceRequest.Stage.INFERENCE, 0, progress_total)
        with Uploader(
            upload_f=_upload_f,
            notify_f=_notify_f,
            exception_handler=_exception_handler,
            logger=api.logger,
        ) as uploader:
            uploader.raise_from_notify
            for fig_i, figure in enumerate(figures, 1):
                figure = api.video.figure._convert_json_info(figure)
                if not figure.geometry_type == Rectangle.geometry_name():
                    raise TypeError(f"Tracking does not work with {figure.geometry_type}.")
                api.logger.info("figure:", extra={"figure": figure._asdict()})
                sly_geometry: Rectangle = deserialize_geometry(
                    figure.geometry_type, figure.geometry
                )
                init = False
                for frame_i, (frame, next_frame) in enumerate(
                    tracker_interface.frames_loader_generator(), 1
                ):
                    target = PredictionBBox(
                        "",  # TODO: can this be useful?
                        [
                            sly_geometry.top,
                            sly_geometry.left,
                            sly_geometry.bottom,
                            sly_geometry.right,
                        ],
                        None,
                    )

                    if not init:
                        self.initialize(frame.image, target)
                        init = True

                    logger.debug("Start prediction")
                    t = time.time()
                    geometry = self.predict(
                        rgb_image=next_frame.image,
                        prev_rgb_image=frame.image,
                        target_bbox=target,
                        settings=self.custom_inference_settings_dict,
                    )
                    logger.debug("Prediction done. Time: %f sec", time.time() - t)
                    sly_geometry = self._to_sly_geometry(geometry)

                    figure_id = uuid.uuid5(namespace=uuid.NAMESPACE_URL, name=f"{time.time()}").hex
                    result_figure = api.video.figure._convert_json_info(
                        {
                            ApiField.ID: figure_id,
                            ApiField.OBJECT_ID: figure.object_id,
                            "meta": {"frame": next_frame.frame_index},
                            ApiField.GEOMETRY_TYPE: sly_geometry.geometry_name(),
                            ApiField.GEOMETRY: sly_geometry.to_json(),
                            ApiField.TRACK_ID: tracker_interface.track_id,
                        }
                    )

                    uploader.put([result_figure])

                    logger.debug(
                        "Frame [%d / %d] processed.",
                        frame_i,
                        tracker_interface.frames_count,
                        extra={
                            "frame_index": frame_i,
                            "figure_index": fig_i,
                            "inference_request_uuid": inference_request.uuid,
                        },
                    )

                    if inference_request.is_stopped() or tracker_interface.is_stopped():
                        if isinstance(tracker_interface.stop_reason(), Exception):
                            raise tracker_interface.stop_reason()
                        api.logger.info(
                            "Inference request stopped.",
                            extra={"inference_request_uuid": inference_request.uuid},
                        )
                        tracker_interface.notify_progress(
                            inference_request.progress.current,
                            inference_request.progress.current,
                            frame_range_asc,
                        )
                        return
                    if uploader.has_exception():
                        raise uploader.exception

                api.logger.info(
                    "Figure [%d, %d] tracked.",
                    fig_i,
                    len(figures),
                    extra={
                        "figure_id": figure.id,
                        "figure_index": fig_i,
                        "inference_request_uuid": inference_request.uuid,
                    },
                )
            api.logger.info(
                "Finished tracking", extra={"inference_request_uuid": inference_request.uuid}
            )
            tracker_interface.notify_progress(
                inference_request.progress.current,
                inference_request.progress.current,
                frame_range_asc,
            )

    def track(self, api: Api, state: Dict, context: Dict):
        fn = self.send_error_data(api, context)(self._track)
        self.inference_requests_manager.schedule_task(fn, api, context)
        return {"message": "Track task started."}

    def track_api(self, api: Api, state: Dict, context: Dict):
        inference_request, future = self.inference_requests_manager.schedule_task(
            self._track_api, api, context
        )
        future.result()
        logger.info(
            "Track-api request processed.", extra={"inference_request_uuid": inference_request.uuid}
        )
        return inference_request.final_result

    def track_api_files(self, files: List[BinaryIO], settings: Dict):
        logger.info("Start tracking with settings:", extra={"settings": settings})
        frame_indexes = list(
            range(settings["frame_index"], settings["frame_index"] + settings["frames"] + 1)
        )
        geometries = map(self._deserialize_geometry, settings["input_geometries"])
        frames = []
        for file, frame_idx in zip(files, frame_indexes):
            img_bytes = file.read()
            frame = sly_image.read_bytes(img_bytes)
            frames.append(frame)
        logger.info("Start tracking.")
        return self._inference(frames, geometries, settings)

    def track_async(self, api: Api, state: Dict, context: Dict):
        batch_size = context.get("batch_size", self.get_batch_size())
        if self.max_batch_size is not None and batch_size > self.max_batch_size:
            raise ValidationError(
                f"Batch size should be less than or equal to {self.max_batch_size} for this model."
            )

        fn = self.send_error_data(api, context)(self._track_async)
        inference_request, future = self.inference_requests_manager.schedule_task(fn, api, context)

        logger.debug(
            "Inference has scheduled from 'track_async' endpoint",
            extra={"inference_request_uuid": inference_request.uuid},
        )
        return {
            "message": "Inference has started.",
            "inference_request_uuid": inference_request.uuid,
        }

    def initialize(self, init_rgb_image: np.ndarray, target_bbox: PredictionBBox) -> None:
        """
        Initializing the tracker with a new object.

        :param init_rgb_image: frame with object
        :type init_rgb_image: np.ndarray
        :param target_bbox: initial bbox
        :type target_bbox: PredictionBBox
        """
        raise NotImplementedError

    def predict(
        self,
        rgb_image: np.ndarray,
        settings: Dict[str, Any],
        prev_rgb_image: np.ndarray,
        target_bbox: PredictionBBox,
    ) -> PredictionBBox:
        """
        SOT prediction

        :param rgb_image: search frame
        :type rgb_image: np.ndarray
        :param settings: model parameters
        :type settings: Dict[str, Any]
        :param init_rgb_image: previous frame with object
        :type init_rgb_image: np.ndarray
        :param target_bbox: bbox added on previous step
        :type target_bbox: PredictionBBox
        :return: predicted annotation
        :rtype: PredictionBBox
        """
        raise NotImplementedError

    def visualize(
        self,
        predictions: List[PredictionBBox],
        images: List[np.ndarray],
        vis_path: str,
        thickness: int = 2,
        classes_whitelist: Optional[List[str]] = None,
    ):
        vis_path = Path(vis_path)

        for i, (pred, image) in enumerate(zip(predictions, images)):
            out_path = vis_path / f"img_{i}.jpg"
            ann = self._predictions_to_annotation(image, [pred], classes_whitelist)
            ann.draw_pretty(
                bitmap=image,
                color=(255, 0, 0),
                thickness=thickness,
                output_path=str(out_path),
                fill_rectangles=False,
            )

    def _to_sly_geometry(self, dto: PredictionBBox) -> Rectangle:
        top, left, bottom, right = dto.bbox_tlbr
        geometry = Rectangle(top=top, left=left, bottom=bottom, right=right)
        return geometry

    def _create_label(self, dto: PredictionBBox) -> Rectangle:
        geometry = self._to_sly_geometry(dto)
        return Label(geometry, ObjClass("", Rectangle))

    def _get_obj_class_shape(self):
        return Rectangle

    def _predictions_to_annotation(
        self,
        image: np.ndarray,
        predictions: List[Prediction],
        classes_whitelist: Optional[List[str]] = None,
    ) -> Annotation:
        labels = []
        for prediction in predictions:
            if (
                not classes_whitelist in (None, "all")
                and prediction.class_name not in classes_whitelist
            ):
                continue
            label = self._create_label(prediction)
            if label is None:
                # for example empty mask
                continue
            if isinstance(label, list):
                for lb in label:
                    lb.status = LabelingStatus.AUTO
                labels.extend(label)
                continue

            label.status = LabelingStatus.AUTO
            labels.append(label)

        # create annotation with correct image resolution
        ann = Annotation(img_size=image.shape[:2])
        ann = ann.add_labels(labels)
        return ann



# import supervisely.volume.parsers
# import supervisely.volume.loaders
# import supervisely.volume.nrrd_encoder

from .volume import (
    convert_nifti_to_nrrd,
    encode,
    get_extension,
    has_valid_ext,
    inspect_dicom_series,
    inspect_nrrd_series,
    is_valid_ext,
    read_dicom_serie_volume,
    read_dicom_serie_volume_np,
    read_dicom_tags,
    read_nrrd_serie_volume,
    read_nrrd_serie_volume_np,
    validate_format,
)



from pathlib import Path

from supervisely.io.json import load_json_file
from supervisely.nn.inference import SessionJSON


def try_set_conf_auto(session: SessionJSON, conf: float):
    conf_names = ["conf", "confidence", "confidence_threshold", "confidence_thresh"]
    default = session.get_default_inference_settings()
    for name in conf_names:
        if name in default:
            session.inference_settings[name] = conf
            return True
    return False


def read_coco_datasets(cocoGt_json, cocoDt_json):
    from pycocotools.coco import COCO  # pylint: disable=import-error

    if isinstance(cocoGt_json, (str, Path)):
        cocoGt_json = load_json_file(str(cocoGt_json))
    if isinstance(cocoDt_json, (str, Path)):
        cocoDt_json = load_json_file(str(cocoDt_json))
    cocoGt = COCO()
    cocoGt.dataset = cocoGt_json
    cocoGt.createIndex()

    # Fix key error in pycocotools
    info = cocoGt.dataset.get("info", None)
    if info is None:
        cocoGt.dataset["info"] = {}
    # ------------------------------ #

    cocoDt = cocoGt.loadRes(cocoDt_json["annotations"])
    return cocoGt, cocoDt



import sys

import supervisely
from supervisely import *

sys.modules['supervisely_lib'] = supervisely

for module_name in list(sys.modules.keys()):
    if module_name.startswith("supervisely."):
        new_name = module_name.replace("supervisely.", "supervisely_lib.", 1)
        sys.modules[new_name] = sys.modules[module_name]



# coding: utf-8
import json
import os
from typing import Callable, List, Literal, Optional, Union

RAISE_IF_NOT_FOUND = True


def flag_from_env(s: str) -> bool:
    """Returns True if passed string is a flag, False otherwise.
    Possible values to set the flag to True:
        - "1"
        - "true"
        - "yes"

    :param s: string to check
    :type s: str
    :return: True if passed string is a flag, False otherwise
    :rtype: bool
    """
    return s.upper() in ["TRUE", "YES", "1"]


def remap_gpu_devices(in_device_ids):
    """
    Working limitation for CUDA
    :param in_device_ids: real GPU devices indexes. e.g.: [3, 4, 7]
    :return: CUDA ordered GPU indexes, e.g.: [0, 1, 2]
    """
    os.environ["CUDA_VISIBLE_DEVICES"] = ",".join(map(str, in_device_ids))
    return list(range(len(in_device_ids)))


def _int_from_env(value):
    if value is None:
        return value
    return int(value)


def _parse_list_from_env(value: str) -> List[str]:
    import ast

    return [str(x).strip() for x in ast.literal_eval(value)]


def _parse_from_env(
    name: str,
    keys: List[str],
    postprocess_fn: Callable,
    default=None,
    raise_not_found=False,
):
    for k in keys:
        if k in os.environ:
            return postprocess_fn(os.environ[k])

    # env not found
    if raise_not_found is True:
        raise KeyError(
            f"{name} is not defined as environment variable. One of the envs has to be defined: {keys}. Learn more in developer portal: https://developer.supervisely.com/getting-started/environment-variables"
        )

    return default


def agent_id(raise_not_found: Optional[bool] = True) -> int:
    """Returns agent id from environment variable using following keys:
        - AGENT_ID

    :param raise_not_found: if True, raises KeyError if agent id is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: agent id
    :rtype: int
    """
    return _parse_from_env(
        name="agent_id",
        keys=["AGENT_ID"],
        postprocess_fn=_int_from_env,
        default=None,
        raise_not_found=raise_not_found,
    )


def agent_storage(raise_not_found: Optional[bool] = True) -> str:
    """Returns path to the agent's storafe from environment variable using following keys:
        - AGENT_STORAGE

    :param raise_not_found: if True, raises KeyError if agent storage is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: path to the agent's storage
    :rtype: str
    """
    return _parse_from_env(
        name="agent_storage",
        keys=["AGENT_STORAGE"],
        postprocess_fn=lambda x: x,
        default=None,
        raise_not_found=raise_not_found,
    )


def team_id(raise_not_found: Optional[bool] = True) -> int:
    """Returns team id from environment variable using following keys:
        - TEAM_ID
        - CONTEXT_TEAMID
        - context.teamId

    :param raise_not_found: if True, raises KeyError if team id is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: team id
    :rtype: int
    """
    return _parse_from_env(
        name="team_id",
        keys=["CONTEXT_TEAMID", "context.teamId", "TEAM_ID"],
        postprocess_fn=lambda x: int(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def workspace_id(raise_not_found: Optional[bool] = True) -> int:
    """Returns workspace id from environment variable using following keys:
        - WORKSPACE_ID
        - CONTEXT_WORKSPACEID
        - context.workspaceId

    :param raise_not_found: if True, raises KeyError if workspace id is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: workspace id
    :rtype: int
    """
    return _parse_from_env(
        name="workspace_id",
        keys=["CONTEXT_WORKSPACEID", "context.workspaceId", "WORKSPACE_ID"],
        postprocess_fn=lambda x: int(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def project_id(raise_not_found: Optional[bool] = True) -> int:
    """Returns project id from environment variable using following keys:
        - PROJECT_ID
        - CONTEXT_PROJECTID
        - context.projectId
        - modal.state.slyProjectId
        - modal.state.inputProjectId

    :param raise_not_found: if True, raises KeyError if project id is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: project id
    :rtype: int
    """
    return _parse_from_env(
        name="project_id",
        keys=[
            "CONTEXT_PROJECTID",
            "context.projectId",
            "modal.state.slyProjectId",
            "PROJECT_ID",
            "modal.state.inputProjectId",
        ],
        postprocess_fn=lambda x: int(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def dataset_id(raise_not_found: Optional[bool] = True) -> int:
    """Returns dataset id from environment variable using following keys:
        - DATASET_ID
        - CONTEXT_DATASETID
        - context.datasetId
        - modal.state.slyDatasetId
        - modal.state.inputDatasetId

    :param raise_not_found: if True, raises KeyError if dataset id is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: dataset id
    :rtype: int
    """
    return _parse_from_env(
        name="dataset_id",
        keys=[
            "CONTEXT_DATASETID",
            "context.datasetId",
            "modal.state.slyDatasetId",
            "DATASET_ID",
            "modal.state.inputDatasetId",
        ],
        postprocess_fn=lambda x: int(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def team_files_folder(raise_not_found: Optional[bool] = True) -> str:
    """Returns path to the team files folder from environment variable using following keys:
        - CONTEXT_SLYFOLDER
        - context.slyFolder
        - modal.state.slyFolder
        - FOLDER
    NOTE: same as folder
    :param raise_not_found: if True, raises KeyError if team files folder is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: path to the team files folder
    :rtype: str
    """
    return _parse_from_env(
        name="team_files_folder",
        keys=[
            "CONTEXT_SLYFOLDER",
            "context.slyFolder",
            "modal.state.slyFolder",
            "FOLDER",
        ],
        postprocess_fn=lambda x: str(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def team_files_folders(raise_not_found: Optional[bool] = True) -> List[str]:
    """Returns paths to the team files folders from environment variable using following keys:
        - CONTEXT_SLYFOLDERS
        - context.slyFolders
        - modal.state.slyFolders
        - FOLDERS
    NOTE: same as team_files_folders
    :param raise_not_found: if True, raises KeyError if team files folders are not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: path to the team files folders
    :rtype: str
    """
    return _parse_from_env(
        name="team_files_folders",
        keys=[
            "CONTEXT_SLYFOLDERS",
            "context.slyFolders",
            "modal.state.slyFolders",
            "FOLDERS",
        ],
        postprocess_fn=_parse_list_from_env,
        default=[],
        raise_not_found=raise_not_found,
    )


def folder(raise_not_found: Optional[bool] = True) -> str:
    """Returns path to the team files folder from environment variable using following keys:
        - CONTEXT_SLYFOLDER
        - context.slyFolder
        - modal.state.slyFolder
        - FOLDER
    NOTE: Same as team_files_folder
    :param raise_not_found: if True, raises KeyError if team files folder is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: path to the team files folder
    :rtype: str
    """
    return team_files_folder(raise_not_found)


def folders(raise_not_found: Optional[bool] = True) -> List[str]:
    """Returns paths to the team files folders from environment variable using following keys:
        - CONTEXT_SLYFOLDERS
        - context.slyFolders
        - modal.state.slyFolders
        - FOLDERS
    NOTE: Same as team_files_folders
    :param raise_not_found: if True, raises KeyError if team files folders are not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: path to the team files folders
    :rtype: str
    """
    return team_files_folders(raise_not_found)


def team_files_file(raise_not_found: Optional[bool] = True) -> str:
    """Returns path to the file in the team files from environment variable using following keys:
        - CONTEXT_SLYFILE
        - context.slyFile
        - modal.state.slyFile
        - FILE

    NOTE: same as file
    :param raise_not_found: if True, raises KeyError if file is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: path to the file in the team files
    :rtype: str
    """
    return _parse_from_env(
        name="team_files_file",
        keys=["CONTEXT_SLYFILE", "context.slyFile", "modal.state.slyFile", "FILE"],
        postprocess_fn=lambda x: str(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def team_files_files(raise_not_found: Optional[bool] = True) -> List[str]:
    """Returns paths to the files in the team files from environment variable using following keys:
        - CONTEXT_SLYFILES
        - context.slyFiles
        - modal.state.slyFiles
        - FILES

    NOTE: same as team_files_file
    :param raise_not_found: if True, raises KeyError if file is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: path to the file in the team files
    :rtype: str
    """
    return _parse_from_env(
        name="team_files_files",
        keys=["CONTEXT_SLYFILES", "context.slyFiles", "modal.state.slyFiles", "FILES"],
        postprocess_fn=_parse_list_from_env,
        default=[],
        raise_not_found=raise_not_found,
    )


def server_address(raise_not_found: Optional[bool] = True) -> str:
    """Returns server address from environment variable using following keys:
        - SERVER_ADDRESS

    :param raise_not_found: if True, raises KeyError if server address is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: server address
    :rtype: str
    """
    return _parse_from_env(
        name="server_address",
        keys=["SERVER_ADDRESS"],
        postprocess_fn=lambda x: str(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def api_token(raise_not_found: Optional[bool] = True) -> str:
    """Returns an API token from environment variable using following keys:
        - API_TOKEN

    :param raise_not_found: if True, raises KeyError if API token is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: API token
    :rtype: str
    """
    return _parse_from_env(
        name="api_token",
        keys=["API_TOKEN"],
        postprocess_fn=lambda x: str(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def spawn_api_token(raise_not_found: Optional[bool] = True) -> str:
    """Returns SPAWN API token from environment variable using following keys:
        - context.spawnApiToken

    :param raise_not_found: if True, raises KeyError if API token is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: API token
    :rtype: str
    """
    return _parse_from_env(
        name="spawn_api_token",
        keys=["context.spawnApiToken"],
        postprocess_fn=lambda x: str(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def file(raise_not_found: Optional[bool] = True) -> str:
    """Returns path to the file in the team files from environment variable using following keys:
        - CONTEXT_SLYFILE
        - context.slyFile
        - modal.state.slyFile
        - FILE

    NOTE: Same as team_files_file
    :param raise_not_found: if True, raises KeyError if file is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: path to the file in the team files
    :rtype: str
    """
    return team_files_file(raise_not_found)


def files(raise_not_found: Optional[bool] = True) -> List[str]:
    """Returns paths to the files in the team files from environment variable using following keys:
        - CONTEXT_SLYFILES
        - context.slyFiles
        - modal.state.slyFiles
        - FILES

    NOTE: Same as team_files_files
    :param raise_not_found: if True, raises KeyError if file is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: path to the file in the team files
    :rtype: str
    """
    return team_files_files(raise_not_found)


def task_id(raise_not_found: Optional[bool] = True) -> int:
    """Returns task id from environment variable using following keys:
        - TASK_ID

    :param raise_not_found: if True, raises KeyError if task id is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: task id
    :rtype: int
    """
    return _parse_from_env(
        name="task_id",
        keys=["TASK_ID"],
        postprocess_fn=lambda x: int(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def user_login(raise_not_found: Optional[bool] = True) -> str:
    """Returns user login from environment variable using following keys:
        - USER_LOGIN
        - CONTEXT_USERLOGIN
        - context.userLogin

    :param raise_not_found: if True, raises KeyError if user login is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: user login
    :rtype: str
    """
    return _parse_from_env(
        name="user_login",
        keys=["USER_LOGIN", "context.userLogin", "CONTEXT_USERLOGIN"],
        postprocess_fn=lambda x: str(x),
        default="user (debug)",
        raise_not_found=raise_not_found,
    )


def app_name(raise_not_found: Optional[bool] = True) -> str:
    """Returns application's name from environment variable using following keys:
        - APP_NAME

    :param raise_not_found: if True, raises KeyError if application's name is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: application's name
    :rtype: str
    """
    return _parse_from_env(
        name="app_name",
        keys=["APP_NAME"],
        postprocess_fn=lambda x: str(x),
        default="Supervisely App (debug)",
        raise_not_found=raise_not_found,
    )


def user_id(raise_not_found: Optional[bool] = True) -> int:
    """Returns user id from environment variable using following keys:
        - USER_ID
        - CONTEXT_USERID
        - context.userId

    :param raise_not_found: if True, raises KeyError if user id is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: user id
    :rtype: int
    """
    return _parse_from_env(
        name="user_id",
        keys=["USER_ID", "context.userId", "CONTEXT_USERID"],
        postprocess_fn=lambda x: int(x),
        default=None,
        raise_not_found=raise_not_found,
    )


def content_origin_update_interval() -> float:
    """Returns interval of updating the content origin from environment variable using following keys:
        - CONTENT_ORIGIN_UPDATE_INTERVAL

    :return: content origin update interval
    :rtype: float
    """
    return _parse_from_env(
        name="content_origin_update_interval",
        keys=["CONTENT_ORIGIN_UPDATE_INTERVAL"],
        postprocess_fn=lambda x: float(x),
        default=0.5,
        raise_not_found=False,
    )


def smart_cache_ttl(
    raise_not_found: Optional[bool] = False, default: Optional[int] = 30 * 60
) -> int:
    """Returns TTL of the smart cache from environment variable using following keys:
        - SMART_CACHE_TTL

    :param raise_not_found: if True, raises KeyError if smart cache TTL is not found in environment variables
    :type raise_not_found: Optional[bool]
    :param default: default value of smart cache TTL
    :type default: Optional[int]
    :return: smart cache TTL
    :rtype: int
    """
    return _parse_from_env(
        name="smart_cache_ttl",
        keys=["SMART_CACHE_TTL"],
        postprocess_fn=lambda x: max(int(x), 1),
        default=default,
        raise_not_found=raise_not_found,
    )


def smart_cache_size(raise_not_found: Optional[bool] = False, default: Optional[int] = 256) -> int:
    """Returns the size of the smart cache from environment variable using following keys:
        - SMART_CACHE_SIZE

    :param raise_not_found: if True, raises KeyError if smart cache size is not found in environment variables
    :type raise_not_found: Optional[bool]
    :default: default value of smart cache size
    :type default: Optional[int]
    :return: smart cache size
    :rtype: int
    """
    return _parse_from_env(
        name="smart_cache_size",
        keys=["SMART_CACHE_SIZE"],
        postprocess_fn=lambda x: max(int(x), 1),
        default=default,
        raise_not_found=raise_not_found,
    )


def smart_cache_container_dir(default: Optional[str] = "/tmp/smart_cache") -> str:
    """Returns a path to the smart cache dir in the container from environment variable using following keys:
        - SMART_CACHE_CONTAINER_DIR

    :param default: default value of smart cache container dir
    :type default: Optional[str]
    :return: path to the smart cache dir in the container
    :rtype: str
    """
    return _parse_from_env(
        name="smart_cache_container_dir",
        keys=["SMART_CACHE_CONTAINER_DIR"],
        default=default,
        raise_not_found=False,
        postprocess_fn=lambda x: x.strip(),
    )


def autostart() -> bool:
    """Returns autostart flag from environment variable using following keys:
        - modal.state.autostart

    :return: autostart flag
    :rtype: bool
    """
    return _parse_from_env(
        name="autostart",
        keys=["modal.state.autostart", "AUTOSTART"],
        default=False,
        raise_not_found=False,
        postprocess_fn=flag_from_env,
    )


def set_autostart(value: Optional[Literal["1", "true", "yes"]] = None) -> None:
    """Set modal.state.autostart env to the given value.
    Possible values to set the autostart to True:
        - "1"
        - "true"
        - "yes"

    To remove the variable, use `value=None`, or omit the argument.

    :param value: value to set the autostart to
    :type value: Optional[Union[Literal["1", "true", "yes"]], None]
    """
    if value is None:
        os.environ.pop("modal.state.autostart", None)
        return

    if not flag_from_env(value):
        raise ValueError("Unknown value for `autostart` env. Use `1`, `true`, `yes` or None.")
    os.environ["modal.state.autostart"] = value


def apps_cache_dir():
    """Returns apps cache directory path from environment variable using following keys:
        - APPS_CACHE_DIR

    :return: apps cache directory path
    :rtype: str
    """
    return _parse_from_env(
        name="apps_cache_dir",
        keys=["APPS_CACHE_DIR"],
        postprocess_fn=lambda x: x,
        default="/apps_cache",
        raise_not_found=False,
    )


def mininum_instance_version_for_sdk() -> str:
    """Returns minimum instance version required by the SDK from environment variable using following
        - MINIMUM_INSTANCE_VERSION_FOR_SDK

    :return: minimum instance version required by the SDK
    :rtype: str
    """
    return _parse_from_env(
        name="sdk_minimum_instance_version",
        keys=["MINIMUM_INSTANCE_VERSION_FOR_SDK"],
        postprocess_fn=lambda x: x,
        raise_not_found=False,
    )


def semaphore_size() -> int:
    """Returns semaphore size from environment variable using following
        - SUPERVISELY_ASYNC_SEMAPHORE

    :return: semaphore size
    :rtype: int
    """
    return _parse_from_env(
        name="semaphore_size",
        keys=["SUPERVISELY_ASYNC_SEMAPHORE"],
        postprocess_fn=lambda x: int(x),
        raise_not_found=False,
    )


def supervisely_server_path_prefix() -> str:
    """Returns routes prefix from environment variable using following
        - SUPERVISELY_SERVER_PATH_PREFIX

    :return: routes prefix
    :rtype: str
    """
    return _parse_from_env(
        name="supervisely_server_path_prefix",
        keys=["SUPERVISELY_SERVER_PATH_PREFIX"],
        postprocess_fn=lambda x: x,
        default="",
        raise_not_found=False,
    )


def supervisely_skip_https_user_helper_check() -> bool:
    """Returns decision to skip `_check_https_redirect` for API from environment variable using following
        - SUPERVISELY_SKIP_HTTPS_USER_HELPER_CHECK"

    :return: decision to skip `_check_https_redirect` for API
    :rtype: bool
    """
    return _parse_from_env(
        name="supervisely_skip_https_user_helper_check",
        keys=["SUPERVISELY_SKIP_HTTPS_USER_HELPER_CHECK"],
        postprocess_fn=flag_from_env,
        default=False,
        raise_not_found=False,
    )


def configure_minimum_instance_version() -> None:
    """
    Configure MINIMUM_INSTANCE_VERSION_FOR_SDK environment variable
    from the latest entry in versions.json file.

    This function should be called during SDK initialization to automatically
    set the minimum required instance version based on the versions.json file.
    """
    from supervisely._utils import get_latest_instance_version_from_json

    latest_version = get_latest_instance_version_from_json()
    if latest_version:
        os.environ["MINIMUM_INSTANCE_VERSION_FOR_SDK"] = latest_version

def app_categories(raise_not_found: Optional[bool] = False) -> list:
    """Returns a list of app categories from environment variable using following keys:
        - APP_CATEGORIES
    :param raise_not_found: if True, raises KeyError if app category is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: app categories
    :rtype: list
    """
    return _parse_from_env(
        name="app_category",
        keys=["APP_CATEGORIES"],
        postprocess_fn=lambda x: json.loads(x),
        default=[],
        raise_not_found=raise_not_found,
    )


def upload_count(raise_not_found: Optional[bool] = False) -> dict:
    """Returns a dictionary of upload counts from environment variable using following
        - UPLOAD_COUNT
    :param raise_not_found: if True, raises KeyError if upload count is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: upload count
    :rtype: dict
    """
    return _parse_from_env(
        name="upload_count",
        keys=["UPLOAD_COUNT"],
        postprocess_fn=lambda x: json.loads(x),
        default={},
        raise_not_found=raise_not_found,
    )


def uploaded_ids(raise_not_found: Optional[bool] = False) -> dict:
    """Returns a dictionary with dataset IDs as keys and lists of uploaded IDs as values from environment variable using following
        - UPLOADED_IDS
    :param raise_not_found: if True, raises KeyError if uploaded IDs is not found in environment variables
    :type raise_not_found: Optional[bool]
    :return: uploaded IDs
    :rtype: dict
    """
    return _parse_from_env(
        name="uploaded_ids",
        keys=["UPLOADED_IDS"],
        postprocess_fn=lambda x: json.loads(x),
        default={},
        raise_not_found=raise_not_found,
    )


def increment_upload_count(dataset_id: int, count: int = 1) -> None:
    """Increments the upload count for the given dataset id by the specified count.

    :param dataset_id: The dataset id to increment the upload count for.
    :type dataset_id: int
    :param count: The amount to increment the upload count by. Defaults to 1.
    :type count: int
    """
    upload_info = upload_count()
    upload_info[str(dataset_id)] = upload_info.get(str(dataset_id), 0) + count
    os.environ["UPLOAD_COUNT"] = json.dumps(upload_info)


def add_uploaded_ids_to_env(dataset_id: int, ids: List[int]) -> None:
    """Adds the list of uploaded IDs to the environment variable for the given dataset ID.

    :param dataset_id: The dataset ID to associate the uploaded IDs with.
    :type dataset_id: int
    :param ids: The list of uploaded IDs to add.
    :type ids: List[int]
    """
    uploaded = uploaded_ids()
    if str(dataset_id) not in uploaded:
        uploaded[str(dataset_id)] = []
    existing_ids = set(uploaded[str(dataset_id)])
    if set(ids).intersection(existing_ids):
        for _id in ids:
            if _id not in existing_ids:
                uploaded[str(dataset_id)].append(_id)
    else:
        uploaded[str(dataset_id)].extend(ids)
    os.environ["UPLOADED_IDS"] = json.dumps(uploaded)



import traceback

import supervisely as sly
from rich.console import Console
from dotenv import load_dotenv
import os


def remove_file_run(team_id: int, path: str) -> bool:
    console = Console()

    api = sly._handle_creds_error_to_console(sly.Api.from_env, console.print)
    if api is False:
        return False

    if api.team.get_info_by_id(team_id) is None:
        console.print(f"\nError: Team with ID={team_id} not exists\n", style="bold red")
        return False

    try:
        api.file.remove_file(team_id, path)
        console.print(f"\nFile '{path}' was successfully removed\n", style="bold green")
        return True

    except:
        console.print(f"\nRemoving file failed\n", style="bold red")
        traceback.print_exc()
        return False


def remove_directory_run(team_id: int, path: str) -> bool:
    console = Console()

    api = sly._handle_creds_error_to_console(sly.Api.from_env, console.print)
    if api is False:
        return False

    if api.team.get_info_by_id(team_id) is None:
        console.print(f"\nError: Team with ID={team_id} doesn't exist\n", style="bold red")
        return False

    try:
        api.file.remove_dir(team_id, path)
        console.print(f"\nDirectory '{path}' has been successfully removed\n", style="bold green")
        return True

    except:
        console.print(f"\nError: Removing directory has been failed\n", style="bold red")
        traceback.print_exc()
        return False
